<?php
/**
 * Cron Functions
 *
 * Schedules events.
 *
 * @package     Restrict Content Pro
 * @subpackage  Cron Functions
 * @copyright   Copyright (c) 2017, Restrict Content Pro
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 */

/**
 * Set up the following cron job events:
 *
 * Daily: Expired users check
 * @see rcp_check_for_expired_users()
 *
 * Daily: Send expiring soon notices
 * @see rcp_check_for_soon_to_expire_users()
 *
 * Daily: Check and update member counts
 * @see rcp_check_member_counts()
 *
 * @return void
 */
function rcp_setup_cron_jobs() {

	if ( ! wp_next_scheduled( 'rcp_expired_users_check' ) ) {
		wp_schedule_event( current_time( 'timestamp' ), 'daily', 'rcp_expired_users_check' );
	}

	if ( ! wp_next_scheduled( 'rcp_send_expiring_soon_notice' ) ) {
		wp_schedule_event( current_time( 'timestamp' ), 'daily', 'rcp_send_expiring_soon_notice' );
	}

	if ( ! wp_next_scheduled( 'rcp_check_member_counts' ) ) {
		wp_schedule_event( current_time( 'timestamp' ), 'daily', 'rcp_check_member_counts' );
	}

	if ( ! wp_next_scheduled( 'rcp_mark_abandoned_payments' ) ) {
		wp_schedule_event( current_time( 'timestamp' ), 'daily', 'rcp_mark_abandoned_payments' );
	}
}
add_action('wp', 'rcp_setup_cron_jobs');

/**
 * Check for expired users
 *
 * Runs each day and checks for expired users. If their account has expired, their status
 * is updated to "expired" and, based on settings, they may receive an email.
 *
 * @see rcp_email_on_expiration()
 *
 * @return void
 */
function rcp_check_for_expired_users() {

	rcp_log( 'Starting rcp_check_for_expired_users() cron job.', true );

	$current_time = date( 'Y-m-d H:i:s', strtotime( '-2 days', current_time( 'timestamp' ) ) );

	$expired_memberships = rcp_get_memberships( array(
		'expiration_date_query' => array(
			'after'  => '0000-00-00 00:00:00',
			'before' => $current_time
		),
		'status' => array( 'active', 'cancelled' ),
		'number' => 99999
	) );

	$expired_memberships = apply_filters( 'rcp_check_for_expired_users_members_filter', $expired_memberships );

	if( $expired_memberships ) {
		foreach( $expired_memberships as $key => $membership ) {

			/**
			 * @var RCP_Membership $membership
			 */

			$expiration_date = $membership->get_expiration_time();
			if( $expiration_date && current_time( 'timestamp' ) > $expiration_date ) {
				rcp_log( sprintf( 'Expiring membership #%d via cron job.', $membership->get_id() ) );
				$membership->set_status( 'expired' );
			}
		}
	} else {
		rcp_log( 'No expired memberships found.' );
	}
}
//add_action( 'admin_init', 'rcp_check_for_expired_users' );
add_action( 'rcp_expired_users_check', 'rcp_check_for_expired_users' );

/**
 * Check for soon-to-expire users
 *
 * Runs each day and checks for members that are soon to expire. Based on settings, each
 * member gets sent an expiry notice email.
 *
 * @uses RCP_Reminders
 *
 * @return void
 */
function rcp_check_for_soon_to_expire_users() {

	rcp_log( 'Starting rcp_check_for_soon_to_expire_users() cron job.', true );

	$reminders = new RCP_Reminders();
	$reminders->send_reminders();

}
add_action( 'rcp_send_expiring_soon_notice', 'rcp_check_for_soon_to_expire_users' );

/**
 * Counts the active members on a membership level to ensure counts are accurate.
 *
 * Runs once per day
 *
 * @since 2.6
 *
 * @return void
 */
function rcp_check_member_counts() {

	rcp_log( 'Starting rcp_check_member_counts() cron job.', true );

	global $rcp_levels_db;
	$levels = $rcp_levels_db->get_levels();

	if( ! $levels ) {
		return;
	}

	$statuses = array( 'active', 'pending', 'cancelled', 'expired', 'free' );

	foreach( $levels as $level ) {

		foreach( $statuses as $status ) {
			$key = $level->id . '_' . $status . '_member_count';

			// Change "free" to "active".
			if ( 'free' == $status ) {
				$status = 'active';
			}

			$count = rcp_count_memberships( array(
				'status'    => $status,
				'object_id' => $level->id
			) );

			$rcp_levels_db->update_meta( $level->id, $key, $count );

		}

	}
}
add_action( 'rcp_check_member_counts', 'rcp_check_member_counts' );

/**
 * Find pending payments that are more than a week old and mark them as abandoned.
 *
 * @since 2.9
 * @return void
 */
function rcp_mark_abandoned_payments() {

	rcp_log( 'Starting rcp_mark_abandoned_payments() cron job.', true );

	/**
	 * @var RCP_Payments $rcp_payments_db
	 */
	global $rcp_payments_db;

	$args = array(
		'fields' => 'id',
		'number' => 9999,
		'status' => 'pending',
		'date'   => array(
			'end' => date( 'Y-m-d', strtotime( '-7 days', current_time( 'timestamp' ) ) )
		)
	);

	$payments = $rcp_payments_db->get_payments( $args );

	if ( $payments ) {
		foreach ( $payments as $payment ) {
			$rcp_payments_db->update( $payment->id, array( 'status' => 'abandoned' ) );
		}
	}

}
add_action( 'rcp_mark_abandoned_payments', 'rcp_mark_abandoned_payments' );
